from typing import Any
from typing import List
from typing import Type
from typing import Callable

from pydantic import VERSION as PYDANTIC_VERSION
from pydantic import Field
from pydantic import BaseModel
from pydantic_core.core_schema import CoreSchema
from pydantic_core.core_schema import with_info_plain_validator_function
from pydantic.json_schema import GetJsonSchemaHandler
from pydantic.json_schema import JsonSchemaValue

from django.core.files.uploadedfile import UploadedFile as DjangoUploadedFile

__all__ = [
    "BaseResponse",
    "UploadedFile",
    "OptionalUploadedFile",
    "OptionalUploadedFiles",
    "RequiredUploadedFile",
    "RequiredUploadedFiles",
]

PYDANTIC_V2 = PYDANTIC_VERSION.startswith("2.")


class BaseResponse(BaseModel):
    code: int = Field(
        default=0,
        description="错误码。正确时为：0。",
    )
    message: str = Field(
        default="OK",
        description="错误消息。正确时为：OK。",
    )
    data: Any = Field(
        default=None,
        description="业务数据。不同接口有不同类型的业务数据。",
    )


class UploadedFile(DjangoUploadedFile):

    @classmethod
    def _validate(
        cls,
        __input_value: Any,
        _: Any,
    ) -> "UploadedFile":
        if not isinstance(__input_value, DjangoUploadedFile):
            raise ValueError(f"Expected UploadFile, received: {type(__input_value)}")
        return __input_value

    if not PYDANTIC_V2:

        @classmethod
        def __modify_schema__(
            cls,
            field_schema: Dict[str, Any],
        ) -> None:
            field_schema.update({"type": "string", "format": "binary"})

    @classmethod
    def __get_pydantic_json_schema__(
        cls,
        core_schema: CoreSchema,
        handler: GetJsonSchemaHandler,
    ) -> JsonSchemaValue:
        return {"type": "string", "format": "binary"}

    @classmethod
    def __get_pydantic_core_schema__(
        cls,
        source: Type[Any],
        handler: Callable[[Any], CoreSchema],
    ) -> CoreSchema:
        return with_info_plain_validator_function(cls._validate)


OptionalUploadedFile = UploadedFile | None
OptionalUploadedFiles = List[UploadedFile] | UploadedFile | None

RequiredUploadedFile = UploadedFile
RequiredUploadedFiles = List[UploadedFile] | UploadedFile



from django.shortcuts import render
from django.http import JsonResponse
from django.contrib.auth.decorators import login_required
from django.urls import reverse_lazy
from django.conf import settings

from .openapi3 import get_docs
from .constants import DJANGO_APIS_OPENAPI_LOGIN_URL_KEY

DJANGO_APIS_OPENAPI_LOGIN_URL = getattr(
    settings,
    DJANGO_APIS_OPENAPI_LOGIN_URL_KEY,
    reverse_lazy("admin:index"),
)


@login_required(login_url=DJANGO_APIS_OPENAPI_LOGIN_URL)
def swagger_ui_view(request):
    """SWAGGER-UI管理界面。"""
    return render(
        request,
        "django_apis/swagger_ui.html",
        {},
    )


@login_required(login_url=DJANGO_APIS_OPENAPI_LOGIN_URL)
def swagger_ui_data(request):
    """SWAGGER-UI数据接口。"""
    docs = get_docs()
    return JsonResponse(
        docs,
        json_dumps_params={
            "ensure_ascii": False,
        },
    )


@login_required(login_url=DJANGO_APIS_OPENAPI_LOGIN_URL)
def swagger_ui_init_js(request):
    """SWAGGER-UI JS脚本。"""
    return render(
        request,
        "django_apis/swagger_ui_init.js",
        {},
        content_type="application/javascript",
    )



from django.urls import path
from . import swagger_ui

urlpatterns = [
    path(
        "docs/",
        swagger_ui.swagger_ui_view,
        name="django_apis_swagger_ui_view",
    ),
    path(
        "docs/init.js",
        swagger_ui.swagger_ui_init_js,
        name="django_apis_swagger_ui_init_js",
    ),
    path(
        "docs/data.json",
        swagger_ui.swagger_ui_data,
        name="django_apis_swagger_ui_data",
    ),
]



DJANGO_APIS_FUNC_PARAMETERS_KEY = "_django_apis_func_parameters_key"
DJANGO_APIS_METHODS_KEY = "_django_apis_methods"
DJANGO_APIS_VIEW_FLAG_KEY = "_django_apis_view_flag"
DJANGO_APIS_APIVIEW_INSTANCE_KEY = "_django_apis_apiview_instance"

DJANGO_APIS_OPENAPI_SERVERS_KEY = "DJANGO_APIS_OPENAPI_SERVERS"
DJANGO_APIS_OPENAPI_SERVERS_DEFAULT = [
    {"url": "http://127.0.0.1:8000", "description": "本地开发环境"},
]

DJANGO_APIS_OPENAPI_TITLE_KEY = "DJANGO_APIS_OPENAPI_TITLE"

DJANGO_APIS_OPENAPI_VERSION_KEY = "DJANGO_APIS_OPENAPI_VERSION"
DJANGO_APIS_OPENAPI_VERSION_DEFAULT = "snapshot"

DJANGO_APIS_OPENAPI_DESCRIPTION_KEY = "DJANGO_APIS_OPENAPI_DESCRIPTION"
DJANGO_APIS_OPENAPI_DESCRIPTION_DEFAULT = "......"


DJANGO_APIS_OPENAPI_SECURITY_DEFINITIONS_KEY = (
    "DJANGO_APIS_OPENAPI_SECURITY_DEFINITIONS"
)

DJANGO_APIS_OPENAPI_SCHEMAS_KEY = "DJANGO_APIS_OPENAPI_SCHEMAS"
DJANGO_APIS_OPENAPI_LOGIN_URL_KEY = "DJANGO_APIS_OPENAPI_LOGIN_URL"
